import { LRLanguage, Language, LanguageSupport } from '@codemirror/language';
import { Input, NodePropSource, TreeFragment, PartialParse, NodeSet, Parser } from "@lezer/common";
// A PGML parser configuration.
declare class PGMLParser extends Parser {
    nodeTypes: Record<string, number>;
    readonly nodeSet: NodeSet;
    // The parser's syntax [node types](https://lezer.codemirror.net/docs/ref/#common.NodeSet).
    constructor(props?: NodePropSource[]);
    createParse(input: Input, fragments: readonly TreeFragment[], ranges: readonly {
        from: number;
        to: number;
    }[]): PartialParse;
    getNodeType(name: string): number;
}
// A PG Text parser configuration.
declare class PGTextParser extends Parser {
    nodeTypes: Record<string, number>;
    readonly nodeSet: NodeSet;
    // The parser's syntax [node types](https://lezer.codemirror.net/docs/ref/#common.NodeSet).
    constructor(props?: NodePropSource[]);
    createParse(input: Input, fragments: readonly TreeFragment[], ranges: readonly {
        from: number;
        to: number;
    }[]): PartialParse;
    getNodeType(name: string): number;
}
type CallableKeyOf<S> = keyof {
    [P in keyof S as S[P] extends (token: string) => void ? P : never]: unknown;
};
interface CombineOptions {
    list?: string | {
        indent: number;
    };
    par?: boolean;
    text?: string;
    indent?: string;
}
interface BlockDefinition {
    type?: string;
    token?: string;
    align?: string;
    allowDblStar?: boolean;
    allowPar?: boolean;
    allowStar?: boolean;
    allowTriStar?: boolean;
    balance?: RegExp;
    breakInside?: boolean;
    cancelNL?: boolean;
    cancelPar?: boolean;
    cancelUnbalanced?: boolean;
    class?: string;
    combine?: CombineOptions;
    container?: string;
    display?: boolean;
    displaystyle?: boolean;
    ignoreIndent?: boolean;
    isContainer?: boolean;
    noIndent?: number;
    options?: string[];
    parseAll?: boolean;
    parseComments?: boolean;
    parseQuoted?: boolean;
    parseSlashes?: boolean;
    parseSubstitutions?: boolean;
    parsed?: boolean;
    terminateMethod?: CallableKeyOf<PGMLParse>;
    terminator?: string | RegExp;
}
declare class PGMLParse {
    indent: number;
    actualIndent: number;
    atLineStart: number;
    atBlockStart: number;
    ignoreNL?: number;
    input: string;
    pos: number;
    block?: Block;
    root?: Root;
    split: (string | undefined)[];
    i: number;
    constructor(string: string);
    Split(input: string): (string | undefined)[];
    isLineEnd(block?: Block): boolean;
    nextChar(defaultNext?: string): string;
    prevChar(defaultPrev?: string): string;
    Parse(split: (string | undefined)[]): void;
    All(token: string): void;
    Begin(token: string, id?: string, options?: {
        terminator?: RegExp;
        bullet?: string;
        n?: number;
    }): void;
    End(action: string | undefined, endAt?: Block): void;
    Terminate(token?: string): void;
    EndContainer(token: string): void;
    Unbalanced(token: string): void;
    Text(text: string, force?: boolean): void;
    Item(type: string, token: string, options?: {
        noIndent?: number;
        options?: string[];
        hasStar?: number;
    }): void;
    Break(token: string): void;
    ForceBreak(token: string): void;
    Par(token: string, endAt?: Block): void;
    Indent(token: string): void;
    Slash(token: string): void;
    Brace(token: string): void;
    Verbatim(token: string): void;
    Image(token: string): void;
    Answer(token: string): void;
    Emphasis(token: string): void;
    Star(token: string): void;
    Rule(token: string): void;
    Bullet(token: string, bullet: string): void;
    Heading(token: string): void;
    Center(token: string): void;
    Align(token: string): void;
    Code(token: string): void;
    Preformatted(token: string): void;
    Quoted(token: string): void;
    Quote(token: string): void;
    NOOP(_token: string): void;
    terminateGetString(_token: string): void;
    terminateBalance(_token: string): void;
    terminateCode(_token: string): void;
    terminatePre(token: string): void;
    terminateOptions(_token: string): void;
    StarOption(token: string): boolean;
    stackText(stack?: (string | Item)[]): string;
    stackString(): string;
    replaceBalance(item: Item): string;
    replaceText(item: Item): string;
    replaceQuote(item: Item): string;
    replaceVariable(item: Item): string;
    replaceCommand(item: Item): string;
}
interface Fields extends BlockDefinition {
    parser?: PGMLParse;
    stack?: (Item | string)[];
    prev?: Block;
    hasStar?: number;
    indent?: number;
    noIndent?: number;
    options?: string[];
    bullet?: string;
    n?: number;
}
declare class Item implements BlockDefinition {
    type: string;
    from: number;
    to: number;
    token?: string;
    stack?: (Item | string)[];
    children?: Item[];
    optionStack?: Item[];
    text?: string;
    textFrom?: number;
    hasWarning?: boolean;
    bullet?: string;
    hasDblStar?: boolean;
    hasStar?: number;
    indent?: number;
    n?: number;
    align?: string;
    allowDblStar?: boolean;
    allowPar?: boolean;
    allowStar?: boolean;
    allowTriStar?: boolean;
    balance?: RegExp;
    breakInside?: boolean;
    cancelNL?: boolean;
    cancelPar?: boolean;
    cancelUnbalanced?: boolean;
    class?: string;
    combine?: CombineOptions;
    container?: string;
    display?: boolean;
    displaystyle?: boolean;
    ignoreIndent?: boolean;
    isContainer?: boolean;
    noIndent?: number;
    options?: string[];
    parseAll?: boolean;
    parseComments?: boolean;
    parseQuoted?: boolean;
    parseSlashes?: boolean;
    parseSubstitutions?: boolean;
    parsed?: boolean;
    terminateMethod?: CallableKeyOf<PGMLParse>;
    terminator?: string | RegExp;
    constructor(type: string, from: number, fields?: Fields);
    pushChild(child: Item): void;
    pushOption(child: Item): void;
    quote(input: string): string;
    stringifyObject(obj: object): string;
    show(indent?: string): string;
}
declare class Block extends Item {
    prev?: Block;
    containerEnd?: string | RegExp;
    constructor(type: string, from: number, fields?: Fields);
    pushText(text: string, force?: boolean): void;
    pushItem(...items: (string | Item)[]): void;
    topItem(i?: number): string | Item;
    popItem(): string | Item | undefined;
    // FIXME: There are a lot of hacks in this method.
    combineTopItems(i?: number): void;
    show(indent?: string): string;
}
declare class Root extends Block {
    parser?: PGMLParse;
    constructor(parser: PGMLParse);
    pushItem(...items: Item[]): void;
}
declare const pgmlShow: (input: string) => string;
declare const pgmlParser: PGMLParser;
declare const pgmlLanguage: Language;
declare const pgml: () => LanguageSupport;
declare const pgTextParser: PGTextParser;
declare const pgTextLanguage: Language;
declare const pgText: () => LanguageSupport;
declare const pgLanguage: LRLanguage;
declare const pg: () => LanguageSupport;
export { pgmlShow, pgmlParser, pgmlLanguage, pgml, pgTextParser, pgTextLanguage, pgText, pgLanguage, pg };
